/*
	808SetClock
	Copyright (C) 2010 Martin Schlosser
	email: neotrantor@users.sourceforge.net
	www:   https://sourceforge.net/users/neotrantor

	808SetClock is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	808SetClock is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with 808SetClock in a file named COPYING.
	If not, see <http://www.gnu.org/licenses/>.
*/

#include <windows.h>
#include <CommCtrl.h>
#include <stdio.h>

#include <resource\resource.h>

//#define _TEST_
#ifndef _TEST_
	#define DRIVES_FILTER
#endif

#define DRIVE_ICON_WIDTH	16
#define DRIVE_ICON_HEIGHT	16

#define FANCY_STYLE
#ifdef FANCY_STYLE
	#define COLOR_TEXT_NORMAL				GetSysColor(COLOR_WINDOWTEXT)
	#define COLOR_BG_NORMAL					RGB(0xB3, 0xC6, 0xFF)
	#define COLOR_TEXT_HIGHLIGHTED	RGB(0x00, 0x11, 0x77)
	#define COLOR_BG_HIGHLIGHTED		RGB(0xFF, 0xDF, 0x80)
	#define COLOR_TEXT_DISABLED			COLOR_TEXT_NORMAL/*GetSysColor(COLOR_GRAYTEXT)*/
	#define COLOR_BG_DISABLED				GetSysColor(COLOR_SCROLLBAR)
	#define COLOR_BORDER						RGB(0xF0, 0xC0, 0x00)
#else
	#define COLOR_TEXT_NORMAL				GetSysColor(COLOR_WINDOWTEXT)
	#define COLOR_BG_NORMAL					GetSysColor(COLOR_WINDOW)
	#define COLOR_TEXT_HIGHLIGHTED	GetSysColor(COLOR_HIGHLIGHTTEXT)
	#define COLOR_BG_HIGHLIGHTED		GetSysColor(COLOR_HIGHLIGHT)
	#define COLOR_TEXT_DISABLED			COLOR_TEXT_NORMAL/*GetSysColor(COLOR_GRAYTEXT)*/
	#define COLOR_BG_DISABLED				GetSysColor(COLOR_SCROLLBAR)
	#define COLOR_BORDER						RGB(0x00, 0x00, 0x00)
#endif

#define APP_TITLE								"808SetClock v1.3.1"
#define NO_REMOVABLEMEDIA_FOUND "No USB media found!"
#define ERROR_CREATEFILE				"Couldn't create file!"

#define URL_HOME								"http://www.software-schlosser.de"
#define URL_DONATE							"https://www.paypal.com/cgi-bin/webscr?cmd=_donations&business=paypal%40wasweissderteufel%2ede&lc=DE&item_name=Software%20Schlosser&cn=Mitteilung%20an%20den%20H%c3%a4ndler&no_shipping=1&currency_code=EUR&bn=PP%2dDonationsBF%3abtn_donate_SM%2egif%3aNonHosted"
#define URL_MODEL								"http://www.chucklohr.com/808/#ComparisonOfVersions"

bool									GetStorageDeviceProperties(	IN const char *lpszDriveRoot,
																									OUT PSTORAGE_DEVICE_DESCRIPTOR lpStorageDeviceDescriptor,
																									IN size_t nStructSize);
void									ShowErrorText(DWORD dwError);

HINSTANCE							g_hInst = NULL;

// ----------------------------------------------------------------------------
bool CreateDateTimeFile(HWND hWnd, const char *lpszDrive, int nModel, bool bHideDate)
// ----------------------------------------------------------------------------
{
	const char *pCamModels[][2] = {
		{ "time.txt",			"%d.%02d.%02d %02d:%02d:%02d"										},		// #1
		{ "SetTime.txt",	"%d.%02d.%02d %02d:%02d:%02d"										},		// #2
//		{ "TAG.txt",			"[date]\r\n%d/%02d/%02d\r\n%02d:%02d:%02d\r\n"	},	// #3
//		{ "TAG.txt",			"[date]  \n%d/%02d/%02d  \n%02d:%02d:%02d  \n"	},		// compatible to #3 and #6
		{ "TAG.txt",			"[date]  \n%d/%02d/%02d  \n%02d:%02d:%02d"			},		// compatible to #3 and #6
		{ "time.txt",			"%d.%02d.%02d %02d:%02d:%02d %c"								},		// #4
		{ "timerset.txt",	"%d%02d%02d%02d%02d%02d %c"											}			// #5
	};

	SYSTEMTIME Date;
	SYSTEMTIME Time;
	if(	SendMessage(GetDlgItem(hWnd, IDC_DATEPICKER), DTM_GETSYSTEMTIME, 0, (LPARAM) &Date) == GDT_VALID &&
			SendMessage(GetDlgItem(hWnd, IDC_TIMEPICKER), DTM_GETSYSTEMTIME, 0, (LPARAM) &Time) == GDT_VALID)
	{
		char szContent[512];
		char szDestFile[MAX_PATH+1];
		strncpy(szDestFile, lpszDrive, 3);
		szDestFile[3] = '\0';
		strncat(szDestFile, pCamModels[nModel][0],	sizeof(szDestFile)-1);
		int nContentLength = _snprintf(szContent, sizeof(szContent)-1, pCamModels[nModel][1],
																	(int) Date.wYear, (int) Date.wMonth, (int) Date.wDay,
																	(int) Time.wHour, (int) Time.wMinute, (int) Time.wSecond,
																	bHideDate ? 'N' : 'Y');

		if(nContentLength > 0)
		{
#ifdef _TEST_
				char szTest[1024];
				_snprintf(szTest, sizeof(szTest)-1, "File: \"%s\"\r\nContent: \"%s\"", szDestFile, szContent);
				MessageBox(hWnd, szTest, APP_TITLE, MB_OK | MB_ICONERROR);
#else
			FILE *lpFile = fopen(szDestFile, "wb");
			if(lpFile != NULL)
			{
				fwrite(szContent, sizeof(char), nContentLength, lpFile);
				fclose(lpFile);
			}
			else
			{
				MessageBox(hWnd, ERROR_CREATEFILE, APP_TITLE, MB_OK | MB_ICONERROR);
				return false;
			}
#endif
		}
	}

	return true;
}

// ----------------------------------------------------------------------------
void DrawComboBoxItem(HWND hWnd, int nDlgItem, LPDRAWITEMSTRUCT lpDrawItemStruct, HICON hIcon)
// ----------------------------------------------------------------------------
{
	// control type must be combobox
	if(lpDrawItemStruct->CtlType != ODT_COMBOBOX)
		return;
	switch(lpDrawItemStruct->itemAction)
	{
		case ODA_FOCUS:
			if((lpDrawItemStruct->itemState & ODS_NOFOCUSRECT) != ODS_NOFOCUSRECT)
				DrawFocusRect(lpDrawItemStruct->hDC, &lpDrawItemStruct->rcItem);
		break;

		case ODA_SELECT:
		case ODA_DRAWENTIRE:
			if(lpDrawItemStruct->itemID == 0xFFFFFFFF)
				break;

			// determine text / bg colors
			DWORD dwColorText	= COLOR_TEXT_NORMAL;
			DWORD dwColorBg		= COLOR_BG_NORMAL;
			if(lpDrawItemStruct->itemState & ODS_SELECTED)
			{
				dwColorText	= COLOR_TEXT_HIGHLIGHTED;
				dwColorBg		= COLOR_BG_HIGHLIGHTED;
			}
			if((lpDrawItemStruct->itemState & (ODS_GRAYED | ODS_DISABLED | ODS_INACTIVE)) != 0)
			{
				dwColorText	= COLOR_TEXT_DISABLED;
				dwColorBg		= COLOR_BG_DISABLED;
			}
			SetTextColor(lpDrawItemStruct->hDC,	dwColorText);
			SetBkColor(lpDrawItemStruct->hDC,		dwColorBg);

#ifdef FANCY_STYLE
			// draw custom background
			HBRUSH	hBrushItemRect	= CreateSolidBrush(dwColorBg);
			HPEN		hPenItemRect		= CreatePen(PS_SOLID, 1, COLOR_BORDER);
			if(lpDrawItemStruct->itemState & ODS_SELECTED)
			{
				HBRUSH	hBrushOld	= (HBRUSH)	SelectObject(lpDrawItemStruct->hDC, hBrushItemRect);
				HPEN		hPenOld		= (HPEN)		SelectObject(lpDrawItemStruct->hDC, hPenItemRect);
				RoundRect(lpDrawItemStruct->hDC, lpDrawItemStruct->rcItem.left+2, lpDrawItemStruct->rcItem.top, lpDrawItemStruct->rcItem.right-2, lpDrawItemStruct->rcItem.bottom, 4, 4);
				SelectObject(lpDrawItemStruct->hDC, hBrushOld);
				SelectObject(lpDrawItemStruct->hDC, hPenOld);
			}
			else
			{
				FillRect(lpDrawItemStruct->hDC, &lpDrawItemStruct->rcItem, hBrushItemRect);
			}
			DeleteObject(hBrushItemRect);
			DeleteObject(hPenItemRect);
#else
			// draw focus rect
			if((lpDrawItemStruct->itemState & ODS_FOCUS) != 0 && (lpDrawItemStruct->itemState & ODS_NOFOCUSRECT) != ODS_NOFOCUSRECT)
				DrawFocusRect(lpDrawItemStruct->hDC, &lpDrawItemStruct->rcItem);
#endif
			// get item text
			char szItemText[MAX_PATH+1];
			SendMessage(GetDlgItem(hWnd, nDlgItem), CB_GETLBTEXT, lpDrawItemStruct->itemID, (LPARAM) szItemText);

			// draw item text
			int nTextOffsetLeft = 6;
			if(hIcon != NULL)
				nTextOffsetLeft += 24;
			ExtTextOut(lpDrawItemStruct->hDC, lpDrawItemStruct->rcItem.left+nTextOffsetLeft, lpDrawItemStruct->rcItem.top+1, 0, &lpDrawItemStruct->rcItem, szItemText, strlen(szItemText), 0);

			// get icon handle (stored in itemdata)
			if(hIcon != NULL)
				DrawIconEx(lpDrawItemStruct->hDC, lpDrawItemStruct->rcItem.left+6, lpDrawItemStruct->rcItem.top+1, hIcon, DRIVE_ICON_WIDTH, DRIVE_ICON_HEIGHT, 0, 0, DI_NORMAL);
		break;
	}
}

typedef int (CALLBACK *tDrivesComboFilter)(const char *);
// ----------------------------------------------------------------------------
int CALLBACK DrivesComboFilter(const char *lpszDriveRoot)
// ----------------------------------------------------------------------------
{
	STORAGE_DEVICE_DESCRIPTOR sdd;
	GetStorageDeviceProperties(lpszDriveRoot, &sdd, sizeof(STORAGE_DEVICE_DESCRIPTOR));

	// TODO: filter by VendorID and ProductID

	if(sdd.BusType == BusTypeUsb)
		return 1;

/*
	switch(GetDriveType(lpszDriveRoot))
	{
		case DRIVE_REMOVABLE:
			return 1;

//		case DRIVE_UNKNOWN:
//		case DRIVE_NO_ROOT_DIR:
//		case DRIVE_FIXED:
//		case DRIVE_REMOTE:
//		case DRIVE_CDROM:
//		case DRIVE_RAMDISK:
//		default:
//			return 0;
	}
*/
	return 0;
}

// ----------------------------------------------------------------------------
bool FillDrivesCombo(HWND hWnd, int nDlgItem, tDrivesComboFilter DrivesComboFilter = NULL)
// ----------------------------------------------------------------------------
{
	DWORD dwRet = 0;

	// get drives string size
	DWORD	dwDriveStringBufferSize	= GetLogicalDriveStrings(0, NULL);
	if(dwDriveStringBufferSize < 0)
		return false;
	// alloc memory for drives string
	char *lpszDriveStrings	= new char[dwDriveStringBufferSize];
	if(lpszDriveStrings != NULL)
	{
		// save pointer for later deletion
		char *lpszTmpDriveStrings = lpszDriveStrings;
		// retrieve drives string
		dwRet = GetLogicalDriveStrings(dwDriveStringBufferSize, lpszTmpDriveStrings);
		// check returned size
		if((dwRet+1) == dwDriveStringBufferSize)
		{
			char szDriveRoot[MAX_PATH+1];
			size_t nPos = 0;
			// format of drives string: "...\0...\0...\0\0"
			while(*lpszTmpDriveStrings != '\0')
			{
				// Read next drive letter in temp buffer
				while(*lpszTmpDriveStrings != '\0')
					szDriveRoot[nPos++] = *lpszTmpDriveStrings++;
				szDriveRoot[nPos] = '\0';
				nPos = 0;
				lpszTmpDriveStrings++;

#ifdef DRIVES_FILTER
				// skip drives A, B, C since we are looking for USB media
				if(szDriveRoot[0] <= 'C')
					continue;
#endif

				char szDriveText[1024];
				char szDriveName[MAX_PATH+1];
				char szDriveFS[MAX_PATH+1];
				if(GetVolumeInformation(szDriveRoot, szDriveName, MAX_PATH, NULL, NULL, NULL, szDriveFS, MAX_PATH) != 0)
				{
					_snprintf(szDriveText, sizeof(szDriveText)-1, "%s %s (%s)", szDriveRoot, szDriveName, szDriveFS);
/*
					char pOutBuf[8192] = { 0 };
					PSTORAGE_DEVICE_DESCRIPTOR lpStorageDeviceDescriptor	= (PSTORAGE_DEVICE_DESCRIPTOR) pOutBuf;
					if(GetStorageDeviceProperties(szDriveRoot, lpStorageDeviceDescriptor, sizeof(pOutBuf)))
						if(lpStorageDeviceDescriptor->VendorIdOffset != 0 || lpStorageDeviceDescriptor->ProductIdOffset != 0)
							_snprintf(szDriveText, sizeof(szDriveText)-1, "%s %s [%s:%s] (%s)", szDriveRoot, szDriveName,
																																									*(lpStorageDeviceDescriptor->VendorIdOffset == 0 ? "XXXX" : ((const char *) (lpStorageDeviceDescriptor->RawDeviceProperties + lpStorageDeviceDescriptor->VendorIdOffset))),
																																									*(lpStorageDeviceDescriptor->ProductIdOffset == 0 ? "XXXX" : ((const char *) (lpStorageDeviceDescriptor->RawDeviceProperties + lpStorageDeviceDescriptor->ProductIdOffset))),
																																									szDriveFS);
*/
				}
				else
				{
					strncpy(szDriveText, szDriveRoot, sizeof(szDriveText)-1);
				}

#ifdef DRIVES_FILTER
				if(DrivesComboFilter != NULL)
					if(DrivesComboFilter(szDriveRoot) != 1)
						continue;
#endif

				// add drive to combo box
				int iItem = SendMessage(GetDlgItem(hWnd, nDlgItem), CB_ADDSTRING, 0, (LPARAM) szDriveText);

				// get icon for drive letter
				SHFILEINFO shfi;
				if(SHGetFileInfo(szDriveRoot, FILE_ATTRIBUTE_NORMAL, &shfi, sizeof(SHFILEINFO), SHGFI_USEFILEATTRIBUTES | SHGFI_ICON | SHGFI_SMALLICON) != 0)
					// store icon handle in itemdata
					SendMessage(GetDlgItem(hWnd, nDlgItem), CB_SETITEMDATA, iItem, (LPARAM) shfi.hIcon);
			}
		}
		delete [] lpszDriveStrings;
	}

	return (dwRet > 0);
}

// ----------------------------------------------------------------------------
INT_PTR CALLBACK DlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
// ----------------------------------------------------------------------------
{
	switch(message)
	{
		case WM_INITDIALOG:
		{
			// set app title
			SetWindowText(hWnd, APP_TITLE);

			// set app icons
			SetClassLong(hWnd, GCL_HICON,		(LONG) LoadIcon(g_hInst, (LPCTSTR) IDI_ICON_CLOCK));
			SetClassLong(hWnd, GCL_HICONSM,	(LONG) LoadIcon(g_hInst, (LPCTSTR) IDI_ICON_CLOCK));

			// init date / time picker
			SYSTEMTIME SystemTime;
			GetLocalTime(&SystemTime);
			SystemTime.wSecond = SystemTime.wMilliseconds = 0;
			SendMessage(GetDlgItem(hWnd, IDC_DATEPICKER), DTM_SETSYSTEMTIME, 0, (LPARAM) &SystemTime);
			SendMessage(GetDlgItem(hWnd, IDC_TIMEPICKER), DTM_SETSYSTEMTIME, 0, (LPARAM) &SystemTime);

			HBITMAP hBitmap;
			if((hBitmap = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_BITMAP_OK))) != NULL)
				SendMessage(GetDlgItem(hWnd, IDOK), BM_SETIMAGE, (WPARAM) IMAGE_BITMAP, (LPARAM) hBitmap);
			if((hBitmap = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_BITMAP_CANCEL))) != NULL)
				SendMessage(GetDlgItem(hWnd, IDCANCEL), BM_SETIMAGE, (WPARAM) IMAGE_BITMAP, (LPARAM) hBitmap);
			if((hBitmap = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_BITMAP_HOME))) != NULL)
				SendMessage(GetDlgItem(hWnd, IDC_BUTTON_HOME), BM_SETIMAGE, (WPARAM) IMAGE_BITMAP, (LPARAM) hBitmap);
			if((hBitmap = LoadBitmap(g_hInst, MAKEINTRESOURCE(IDB_BITMAP_DONATE))) != NULL)
				SendMessage(GetDlgItem(hWnd, IDC_BUTTON_DONATE), BM_SETIMAGE, (WPARAM) IMAGE_BITMAP, (LPARAM) hBitmap);

			// drives combo
			FillDrivesCombo(hWnd, IDC_COMBO_DRIVES, DrivesComboFilter);
			if(SendMessage(GetDlgItem(hWnd, IDC_COMBO_DRIVES), CB_GETCOUNT, 0, 0) == 0)
			{
				EnableWindow(GetDlgItem(hWnd, IDC_COMBO_DRIVES), FALSE);
//				EnableWindow(GetDlgItem(hWnd, IDOK), FALSE);
				ShowWindow(GetDlgItem(hWnd, IDOK), SW_HIDE);
				SendMessage(GetDlgItem(hWnd, IDC_COMBO_DRIVES), CB_ADDSTRING, 0, (LPARAM) NO_REMOVABLEMEDIA_FOUND);
			}
			SendMessage(GetDlgItem(hWnd, IDC_COMBO_DRIVES), CB_SETCURSEL, 0, 0);


			// models combo
			int iItem, iDefaultItem;
			iItem = SendMessage(GetDlgItem(hWnd, IDC_COMBO_MODELS), CB_ADDSTRING, 0, (LPARAM) "#1");
			SendMessage(GetDlgItem(hWnd, IDC_COMBO_MODELS), CB_SETITEMDATA, iItem, (LPARAM) 0);
			iItem = SendMessage(GetDlgItem(hWnd, IDC_COMBO_MODELS), CB_ADDSTRING, 0, (LPARAM) "#2");
			SendMessage(GetDlgItem(hWnd, IDC_COMBO_MODELS), CB_SETITEMDATA, iItem, (LPARAM) 1);
			iDefaultItem = iItem = SendMessage(GetDlgItem(hWnd, IDC_COMBO_MODELS), CB_ADDSTRING, 0, (LPARAM) "#3 / #6");
			SendMessage(GetDlgItem(hWnd, IDC_COMBO_MODELS), CB_SETITEMDATA, iItem, (LPARAM) 2);
			iItem = SendMessage(GetDlgItem(hWnd, IDC_COMBO_MODELS), CB_ADDSTRING, 0, (LPARAM) "#4");
			SendMessage(GetDlgItem(hWnd, IDC_COMBO_MODELS), CB_SETITEMDATA, iItem, (LPARAM) 3);
			iItem = SendMessage(GetDlgItem(hWnd, IDC_COMBO_MODELS), CB_ADDSTRING, 0, (LPARAM) "#5");
			SendMessage(GetDlgItem(hWnd, IDC_COMBO_MODELS), CB_SETITEMDATA, iItem, (LPARAM) 4);
			iItem = SendMessage(GetDlgItem(hWnd, IDC_COMBO_MODELS), CB_ADDSTRING, 0, (LPARAM) "I'm not sure");
			SendMessage(GetDlgItem(hWnd, IDC_COMBO_MODELS), CB_SETITEMDATA, iItem, (LPARAM) 5);
			SendMessage(GetDlgItem(hWnd, IDC_COMBO_MODELS), CB_SETCURSEL, iDefaultItem, 0);

			ShowWindow(GetDlgItem(hWnd, IDC_CHECKBOX_DATETIME_HIDE), SW_HIDE);
		}
		break;

		case WM_COMMAND:
			switch(LOWORD(wParam))
			{
				case IDOK:
				{
					int iItemDrive = SendMessage(GetDlgItem(hWnd, IDC_COMBO_DRIVES), CB_GETCURSEL, 0, 0);
					if(iItemDrive == -1)
						return FALSE;
					char szDrive[MAX_PATH+1];
					SendMessage(GetDlgItem(hWnd, IDC_COMBO_DRIVES), CB_GETLBTEXT, iItemDrive, (LPARAM) szDrive);
					if(strchr(szDrive, ':') == NULL)
						return FALSE;
					bool bHideDate	= (SendMessage(GetDlgItem(hWnd, IDC_CHECKBOX_DATETIME_HIDE), BM_GETCHECK, 0, 0) == BST_CHECKED);

					int iItemModel	= SendMessage(GetDlgItem(hWnd, IDC_COMBO_MODELS), CB_GETCURSEL, 0, 0);
					int nModel			= SendMessage(GetDlgItem(hWnd, IDC_COMBO_MODELS), CB_GETITEMDATA, iItemModel, 0);
					if(nModel < 0 || nModel >= 5)
					{
						for(int i=0; i<5; i++)
							if(!CreateDateTimeFile(hWnd, szDrive, i, bHideDate))
								return FALSE;
					}
					else if(!CreateDateTimeFile(hWnd, szDrive, nModel, bHideDate))
						return FALSE;
#ifdef _TEST_
					return FALSE;
#endif
				}

				case IDCANCEL:
					EndDialog(hWnd, wParam);
					return TRUE;
				break;

				case IDC_BUTTON_HOME:
					ShellExecute(NULL, "open", URL_HOME, NULL, NULL, SW_SHOWNORMAL);
				break;

				case IDC_BUTTON_DONATE:
					ShellExecute(NULL, "open", URL_DONATE, NULL, NULL, SW_SHOWNORMAL);
				break;

				case IDC_BUTTON_HELP_MODEL:
					ShellExecute(NULL, "open", URL_MODEL, NULL, NULL, SW_SHOWNORMAL);
				break;

				case IDC_COMBO_MODELS:
					if(HIWORD(wParam) == CBN_SELCHANGE)
					{
						int nCurSel = SendMessage(GetDlgItem(hWnd, IDC_COMBO_MODELS), CB_GETCURSEL, 0, 0);
						ShowWindow(GetDlgItem(hWnd, IDC_CHECKBOX_DATETIME_HIDE), ((nCurSel >= 3 && nCurSel <= 5) ? SW_SHOW : SW_HIDE));
					}
				break;
			}
		break;

		case WM_MEASUREITEM:
		{
			LPMEASUREITEMSTRUCT lpMIS = (LPMEASUREITEMSTRUCT) lParam;
			switch(lpMIS->CtlID)
			{
				case IDC_COMBO_DRIVES:
					lpMIS->itemHeight = DRIVE_ICON_HEIGHT;
				break;
			}
		}
		break;

		case WM_DRAWITEM:
			switch(LOWORD(wParam))
			{
				case IDC_COMBO_DRIVES:
				{
					HICON hIcon = (HICON) SendMessage(GetDlgItem(hWnd, LOWORD(wParam)), CB_GETITEMDATA, ((LPDRAWITEMSTRUCT) lParam)->itemID, 0);
					DrawComboBoxItem(hWnd, LOWORD(wParam), (LPDRAWITEMSTRUCT) lParam, hIcon);
				}
				break;
				case IDC_COMBO_MODELS:
					DrawComboBoxItem(hWnd, LOWORD(wParam), (LPDRAWITEMSTRUCT) lParam, (HICON) NULL);
				break;
			}
		break;

		case WM_CLOSE:
			EndDialog(hWnd, 0);
			return FALSE;
		break;
	}

	return FALSE;
}

// ----------------------------------------------------------------------------
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
// ----------------------------------------------------------------------------
{
	g_hInst = hInstance;
	if(DialogBox(hInstance, MAKEINTRESOURCE(IDD_DIALOG_MAIN), GetDesktopWindow(), reinterpret_cast<DLGPROC> (DlgProc)) < 0)
		ShowErrorText(GetLastError());

	return FALSE;
}

// ----------------------------------------------------------------------------
bool GetStorageDeviceProperties(IN const char *lpszDriveRoot, OUT PSTORAGE_DEVICE_DESCRIPTOR lpStorageDeviceDescriptor, IN size_t nStructSize)
// ----------------------------------------------------------------------------
{
	// szPath without trailing backslash like
	//  "\\\\.\\X:"
	//  "\\\\\?\\Volume{433619ed-c6ea-11d9-a3b2-806d6172696f}
	//  "\\\\.\\PhysicalDrive0"
	char szPath[7] = { 0 };
	strncpy(szPath, "\\\\.\\", sizeof(szPath)-1);
	strncpy(&szPath[4], lpszDriveRoot, sizeof(szPath)-1-4);

	HANDLE hDevice = CreateFile(szPath, 0, FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, NULL);
	if(hDevice != INVALID_HANDLE_VALUE)
	{
		DWORD										dwOutBytes = 0;	// IOCTL output length
		STORAGE_PROPERTY_QUERY	Query;					// input param for query

		// specify the query type
		Query.PropertyId	= StorageDeviceProperty;
		Query.QueryType		= PropertyStandardQuery;

		lpStorageDeviceDescriptor->Size	 = nStructSize;

		// Query using IOCTL_STORAGE_QUERY_PROPERTY 
		BOOL bRes = DeviceIoControl(hDevice,																										// device handle
																IOCTL_STORAGE_QUERY_PROPERTY,																// info of device property
																&Query, sizeof(STORAGE_PROPERTY_QUERY),											// input data buffer
																lpStorageDeviceDescriptor, lpStorageDeviceDescriptor->Size,	// output data buffer
																&dwOutBytes,																								// out's length
																(LPOVERLAPPED) NULL);

		CloseHandle(hDevice);

		return (bRes == TRUE);
	}

	return false;
}

// ----------------------------------------------------------------------------
void ShowErrorText(DWORD dwError)
// ----------------------------------------------------------------------------
{
	LPVOID lpMsgBuf;
	FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
								NULL, dwError, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
								(LPTSTR) &lpMsgBuf, 0, NULL);

	MessageBox(NULL, (LPCTSTR) lpMsgBuf, APP_TITLE, MB_OK | MB_ICONERROR);

	LocalFree(lpMsgBuf);
}
